@@ -37,6 +37,7 @@ setLocalTranslate('cl_core3', sys.modules[__name__])
 __ = getLazyLocalTranslate(_)
 
 from calculate.lib.utils.files import (process, readFile, processProgress,
+                                       readLinesFile,
                                        makeDirectory, getProgPath)
 from calculate.lib.datavars import DataVarsError, CriticalError, Variable
 from calculate.lib.utils.content import getCfgFiles
@@ -242,7 +243,7 @@ class ActionError(Exception):
     pass
 
 
-class Tasks:
+class Tasks(object):
     """
     Класс для создания проверок необходимости запуска задачи в зависимости
     от результатра работы предыдущих задач
@@ -251,39 +252,43 @@ class Tasks:
     def __init__(self, check):
         self.check = check
 
-    def __call__(self, result):
-        return self.check(result)
+    def __call__(self, result, all_result):
+        return self.check(result, all_result)
 
     def __or__(self, y):
-        return Tasks(lambda result: self(result) or y(result))
+        return Tasks(lambda result, all_result: self(result, all_result) or y(result, all_result))
 
     def __ror__(self, y):
-        return Tasks(lambda result: y(result) or self(result))
+        return Tasks(lambda result, all_result: y(result, all_result) or self(result, all_result))
 
     def __and__(self, y):
-        return Tasks(lambda result: self(result) and y(result))
+        return Tasks(lambda result, all_result: self(result, all_result) and y(result, all_result))
 
     def __rand__(self, y):
-        return Tasks(lambda result: y(result) and self(result))
+        return Tasks(lambda result, all_result: y(result, all_result) and self(result, all_result))
 
     def __invert__(self):
-        return Tasks(lambda result: not self(result))
+        return Tasks(lambda result, all_result: not self(result, all_result))
+
+    @classmethod
+    def _result(self, result, all_result):
+        return result
 
     @classmethod
     def success_all(cls, *tasks):
         """
         Все указанные задачи выполнены и выполнены без ошибок
         """
-        return Tasks(
-            lambda result: all(x in result and result[x] for x in tasks))
+        return cls(
+            lambda *args: all(x in cls._result(*args) and cls._result(*args)[x] for x in tasks))
 
     @classmethod
     def success_one_of(cls, *tasks):
         """
         Хотя бы одна из задач выполнена и хотя бы одна из выполненных без ошибок
         """
-        return Tasks(
-            lambda result: any(result[x] for x in tasks if x in result))
+        return cls(
+            lambda *args: any(cls._result(*args)[x] for x in tasks if x in cls._result(*args)))
 
     @classmethod
     def success(cls, inessential=()):
@@ -291,8 +296,8 @@ class Tasks:
         Все ранее запущенные задачи успешно завершены, результат задач
         inessential не важен
         """
-        return Tasks(lambda result: all(result[x] for x in result
-                                        if x not in inessential))
+        return cls(lambda *args: all(cls._result(*args)[x] for x in cls._result(*args)
+                                     if x not in inessential))
 
     @classmethod
     def failed(cls, inessential=()):
@@ -300,7 +305,7 @@ class Tasks:
         Хотя бы одна из задач завершилась неудачно, результат задач
         inessential не важен
         """
-        return Tasks(lambda result: any(not result[x] for x in result
+        return cls(lambda *args: any(not cls._result(*args)[x] for x in cls._result(*args)
                                         if x not in inessential))
 
     @classmethod
@@ -308,49 +313,54 @@ class Tasks:
         """
         Выполнена хотя бы одна задача и все те, которые выполнены с ошибкой
         """
-        return Tasks(
-            lambda result: any(not result[x] for x in tasks if x in result))
+        return cls(
+            lambda *args: any(not cls._result(*args)[x] for x in tasks if x in cls._result(*args)))
 
     @classmethod
     def failed_one_of(cls, *tasks):
         """
         Хотя бы одна из указанных задач выполнена и выполнена с ошибкой
         """
-        return Tasks(
-            lambda result: any(x in result and not result[x] for x in tasks))
+        return cls(
+            lambda *args: any(x in cls._result(*args) and not cls._result(*args)[x] for x in tasks))
 
     @classmethod
     def has(cls, *tasks):
         """
         Был запуск всех перечисленных задач
         """
-        return Tasks(lambda result: all(x in result for x in tasks))
+        return cls(lambda *args: all(x in cls._result(*args) for x in tasks))
 
     @classmethod
     def hasnot(cls, *tasks):
         """
         Не было запуска ни одной из перечисленных задач
         """
-        return Tasks(lambda result: all(x not in result for x in tasks))
+        return cls(lambda *args: all(x not in cls._result(*args) for x in tasks))
 
     @classmethod
     def result(cls, task, eq=None, ne=None):
         if eq:
-            wrapper = lambda result: task in result and result[task] == eq
+            wrapper = lambda *args: task in cls._result(*args) and cls._result(*args)[task] == eq
         elif ne:
-            wrapper = lambda result: task not in result or result[task] != ne
+            wrapper = lambda *args: task not in cls._result(*args) or cls._result(*args)[task] != ne
         else:
-            wrapper = lambda result: task in result and result[task]
-        return Tasks(wrapper)
+            wrapper = lambda *args: task in cls._result(*args) and cls._result(*args)[task]
+        return cls(wrapper)
 
     @classmethod
     def has_any(cls, *tasks):
         """
         Был запуск любой из задач
         """
-        return Tasks(lambda result: any(x in result for x in tasks))
+        return cls(lambda *args: any(x in cls._result(*args) for x in tasks))
 
 
+class AllTasks(Tasks):
+    @classmethod
+    def _result(cls, result, all_result):
+        return all_result
+
 class Action(MethodsInterface):
     """
     Класс для реализации выполнения действия
@@ -543,16 +553,28 @@ class Action(MethodsInterface):
             return print_func, self.formatMessage(self.clVars, action[field])
         return None, None
 
-    def get_tasks(self, result):
+    def get_tasks(self, tasks, result, all_result):
         """
         Герератор задач (поддержка линейной обработки задач в группах)
         """
-        for task in self.tasks:
+        for task in tasks:
             if "group" in task or "tasks" in task:
-                if all(self.get_condition_context(task, result).values()):
+                if all(self.get_condition_context(task, result,
+                                                  all_result).values()):
                     self.group_name = task.get("group", "")
-                    for action in task["tasks"]:
-                        yield action
+                    if "while" in task:
+                        depend = task.get("while", [])
+                        depend = (depend
+                                  if type(depend) in (list, tuple) else [depend])
+                        depend.append(~Tasks.has_any("interrupt"))
+                        while all([x(result, all_result) for x in depend]):
+                            for action in self.get_tasks(task["tasks"],
+                                                         result, all_result):
+                                yield action
+                    else:
+                        for action in self.get_tasks(task["tasks"], result,
+                                                     all_result):
+                            yield action
                     if not self.group_name:
                         self.endGroup()
                     else:
@@ -560,7 +582,7 @@ class Action(MethodsInterface):
             else:
                 yield task
 
-    def get_condition_context(self, action, result):
+    def get_condition_context(self, action, result, all_result):
         """
         Получить результаты проверки по зависимосятм и условиям
         """
@@ -578,7 +600,7 @@ class Action(MethodsInterface):
                       if type(depend) in (list, tuple) else [depend])
             if group:
                 depend.append(Tasks.success_all(group))
-        depend_result = all([x(result) for x in depend])
+        depend_result = all([x(result, all_result) for x in depend])
         # проверить по условиям
         if depend_result:
             condition_funcs = action.get("condition", lambda dv: True)
@@ -599,6 +621,7 @@ class Action(MethodsInterface):
                 pass
 
         result = {}
+        all_result = {}
 
         self.group_name = ""
         self.clVars = dv
@@ -618,7 +641,7 @@ class Action(MethodsInterface):
             self.beginFrame()
             logger.info("Start {methodname}".format(
                 methodname=self.method_name))
-            for action in self.get_tasks(result):
+            for action in self.get_tasks(self.tasks, result, all_result):
                 foreach = action.get("foreach", "")
                 if foreach:
                     foreach = self.clVars.Get(foreach)
@@ -633,7 +656,8 @@ class Action(MethodsInterface):
                     task = False
                     self.clVars.Set('cl_task_name', name, force=True)
                     try:
-                        run_context = self.get_condition_context(action, result)
+                        run_context = self.get_condition_context(action, result,
+                                                                 all_result)
                         actinfo = "Run" if all(run_context.values()) else "Skip"
                         logger.info(
                             "{action} {name}: condition: {condition}, "
@@ -650,6 +674,7 @@ class Action(MethodsInterface):
                         if (run_context['depend'] and
                                 not run_context['condition'] and elseMessage):
                             if "else_error" in action:
+                                all_result[name] = False
                                 if action.get("essential", True):
                                     result[name] = False
                             elsePrint(elseMessage)
@@ -661,9 +686,10 @@ class Action(MethodsInterface):
                             printFunc, message = self.getFormatMessage(
                                 action, "error", "warning", "message")
                             if "confirm" in action and message:
-                                result[name] = \
+                                all_result[name] = \
                                     self.askConfirm(str(message),
                                                     action["confirm"])
+                                result[name] = all_result[name]
                                 continue
                             elif message:
                                 # если действие с командой
@@ -695,9 +721,13 @@ class Action(MethodsInterface):
                                     self.endFrame()
                                     return False
                                 except self.native_error as e:
+                                    if action.get('essential', True):
+                                        printerror = self.printERROR
+                                    else:
+                                        printerror = self.printWARNING
                                     if hasattr(e, "addon") and e.addon:
-                                        self.printERROR(str(e.addon))
-                                    self.printERROR(str(e))
+                                        printerror(str(e.addon))
+                                    printerror(str(e))
                                     res = False
                                 except Exception:
                                     error = shortTraceback(*sys.exc_info())
@@ -719,6 +749,7 @@ class Action(MethodsInterface):
                                     for line in filter(None, lineCmd):
                                         self.printERROR(line)
                                 res = cmd.success()
+                            all_result[name] = res
                             if action.get("essential", True):
                                 result[name] = res
                             failedPrint, failedMessage = (
@@ -738,17 +769,24 @@ class Action(MethodsInterface):
                                     # else:
                                     #    print "[-] Skip ",name
                     except KeyboardInterrupt:
+                        all_result[name] = False
                         if action.get("essential", True):
                             result[name] = False
                         self.endTask(False)
                         self.printWARNING(_("Task interrupted"))
+                        all_result["interrupt"] = False
                         result["interrupt"] = False
                         logger.info("{name}: Interrupeted".format(name=name))
                     except self.native_error as e:
+                        if action.get('essential', True):
+                            printerror = self.printERROR
+                        else:
+                            printerror = self.printWARNING
                         if hasattr(e, "addon") and e.addon:
-                            self.printERROR(str(e.addon))
-                        self.printERROR(str(e))
+                            printerror(str(e.addon))
+                        printerror(str(e))
                         result[name] = False
+                        all_result[name] = False
                         logger.info("{name}: Native error".format(name=name))
                     except CriticalError as e:
                         self.printERROR(str(e))
@@ -756,6 +794,7 @@ class Action(MethodsInterface):
                         return False
                     except BaseException as e:
                         result[name] = False
+                        all_result[name] = False
                         error = shortTraceback(*sys.exc_info())
                         self.printERROR("%s:%s" % (name, error))
                         logger.info("{name}: Unknown exception {exp}".format(
@@ -1491,8 +1530,6 @@ def create_symlink(data_path, old_data_path):
         print _('Symlink %s created') % link_path
     fd.close()
 
-    from calculate.lib.utils.files import readLinesFile
-
     temp_text_file = ''
     for line in readLinesFile(symlinks_file):
         cmdname = os.path.basename(line)
@@ -1577,7 +1614,6 @@ class WsdlMeta(type):
         """
         Создание передаваемой структуры данных для WSDL
         """
-
         def type_convert(s):
             if "bool" in s:
                 return Boolean
@@ -1589,6 +1625,7 @@ class WsdlMeta(type):
                 return String
 
         d = {}
+        d["cl_console_args"] = Array(String)
         if kwargs['datavars'] in WsdlMeta.datavars:
             dv = WsdlMeta.datavars[kwargs['datavars']]
         else:
@@ -1689,6 +1726,7 @@ class WsdlMeta(type):
                         mcs.modify_datavars(dv, kwargs['guivars'])
                     if params.clienttype != 'gui' and "consolevars" in kwargs:
                         mcs.modify_datavars(dv, kwargs['consolevars'])
+                    dv.Set('main.cl_client_type', params.clienttype, force=True)
             else:
                 # костыль для метода install, который меняет локализацию
                 # интрефейса в зависимости от выбранного параметра lang
@@ -1724,26 +1762,31 @@ class WsdlMeta(type):
 
         def wrapper(self, dv=None, params=None):
             if not dv:
-                dv = DataVars()
-                dv.importVariables()
-                dv.importVariables(
+                _dv = DataVars()
+                _dv.importVariables()
+                _dv.importVariables(
                     'calculate.%s.variables' % kwargs['datavars'])
-                dv.defaultModule = kwargs['datavars']
-                dv.flIniFile()
+                _dv.defaultModule = kwargs['datavars']
+                _dv.flIniFile()
                 if params and params.help_set:
-                    dv.Set('cl_help_set', "on", force=True)
+                    _dv.Set('cl_help_set', "on", force=True)
                 if params and params.dispatch_usenew:
-                    dv.Set('cl_dispatch_conf', "usenew", force=True)
-                mcs.modify_datavars(dv, kwargs['setvars'])
+                    _dv.Set('cl_dispatch_conf', "usenew", force=True)
+                if params and params.conargs:
+                    _dv.Set('cl_console_args', params.conargs, force=True)
+            else:
+                _dv = dv
             # созданием группы переменных из datavars согласно параметрам groups
             for groupfunc in kwargs['groups']:
-                groupfunc(dv.addGroup)
+                groupfunc(_dv.addGroup)
+            if not dv:
+                mcs.modify_datavars(_dv, kwargs['setvars'])
             # указание brief если нужно
             if "brief" in kwargs:
-                dv.addBrief(
+                _dv.addBrief(
                     next_label=str(kwargs['brief'].get('next', _('Next'))),
                     image=kwargs['brief'].get('image', None))
-            return dv
+            return _dv
 
         return wrapper
 
