@@ -208,6 +208,7 @@ def local_method(metaclass, args, unknown_args):
             view_obj.brief = None
             view_obj.onlyhelp = True
             view_obj.help_set = True
+            view_obj.conargs = [x[0] for x in args._get_kwargs() if x[1] is not None]
             view_obj.dispatch_usenew = \
                 force_param
             try:
@@ -277,6 +278,7 @@ def call_method(metaObject, args, unknown_args, colorPrint):
         view_obj.brief = None
         view_obj.onlyhelp = True
         view_obj.help_set = False
+        view_obj.conargs = [x for x in unknown_args]
         view_obj.dispatch_usenew = dispatch_usenew
 
         try:
@@ -349,6 +351,7 @@ def call_method(metaObject, args, unknown_args, colorPrint):
         view_obj.brief = True
         view_obj.onlyhelp = False
         view_obj.help_set = False
+        view_obj.conargs = [x[0] for x in args._get_kwargs() if x[1] is not None]
         view_obj.dispatch_usenew = dispatch_usenew
         try:
             view = getattr(metaObject, method_view_name)(0, view_obj)
